@@ -4,28 +4,28 @@ import moderngl
 
 from functools import reduce, wraps
 
-from manimlib.constants import *
-from manimlib.mobject.mobject import Mobject
-from manimlib.mobject.mobject import Point
-from manimlib.utils.bezier import bezier
-from manimlib.utils.bezier import get_smooth_quadratic_bezier_handle_points
-from manimlib.utils.bezier import get_smooth_cubic_bezier_handle_points
-from manimlib.utils.bezier import get_quadratic_approximation_of_cubic
-from manimlib.utils.bezier import interpolate
-from manimlib.utils.bezier import integer_interpolate
-from manimlib.utils.bezier import partial_quadratic_bezier_points
-from manimlib.utils.color import rgb_to_hex
-from manimlib.utils.iterables import make_even
-from manimlib.utils.iterables import resize_array
-from manimlib.utils.iterables import resize_with_interpolation
-from manimlib.utils.iterables import listify
-from manimlib.utils.space_ops import angle_between_vectors
-from manimlib.utils.space_ops import cross2d
-from manimlib.utils.space_ops import earclip_triangulation
-from manimlib.utils.space_ops import get_norm
-from manimlib.utils.space_ops import get_unit_normal
-from manimlib.utils.space_ops import z_to_vector
-from manimlib.shader_wrapper import ShaderWrapper
+from ...constants import *
+from ...mobject.mobject import Mobject
+from ...mobject.mobject import Point
+from ...utils.bezier import bezier
+from ...utils.bezier import get_smooth_quadratic_bezier_handle_points
+from ...utils.bezier import get_smooth_cubic_bezier_handle_points
+from ...utils.bezier import get_quadratic_approximation_of_cubic
+from ...utils.bezier import interpolate
+from ...utils.bezier import integer_interpolate
+from ...utils.bezier import partial_quadratic_bezier_points
+from ...utils.color import rgb_to_hex
+from ...utils.iterables import make_even
+from ...utils.iterables import resize_array
+from ...utils.iterables import resize_with_interpolation
+from ...utils.iterables import listify
+from ...utils.space_ops import angle_between_vectors
+from ...utils.space_ops import cross2d
+from ...utils.space_ops import earclip_triangulation
+from ...utils.space_ops import get_norm
+from ...utils.space_ops import get_unit_normal
+from ...utils.space_ops import z_to_vector
+from ...shader_wrapper import ShaderWrapper
 
 
 class VMobject(Mobject):
@@ -54,7 +54,6 @@ class VMobject(Mobject):
         "joint_type": "auto",
         "flat_stroke": False,
         "render_primitive": moderngl.TRIANGLES,
-        "triangulation_locked": False,
         "fill_dtype": [
             ('point', np.float32, (3,)),
             ('unit_normal', np.float32, (3,)),
@@ -128,7 +127,7 @@ class VMobject(Mobject):
                 if isinstance(width, np.ndarray):
                     arr = width.reshape((len(width), 1))
                 else:
-                    arr = np.array([[w] for w in listify(width)])
+                    arr = np.array([[w] for w in listify(width)], dtype=float)
                 mob.data['stroke_width'] = arr
 
         if background is not None:
@@ -150,6 +149,7 @@ class VMobject(Mobject):
                   stroke_opacity=None,
                   stroke_rgba=None,
                   stroke_width=None,
+                  stroke_background=True,
                   gloss=None,
                   shadow=None,
                   recurse=True):
@@ -164,13 +164,17 @@ class VMobject(Mobject):
 
         if stroke_rgba is not None:
             self.data['stroke_rgba'] = resize_with_interpolation(stroke_rgba, len(fill_rgba))
-            self.set_stroke(width=stroke_width)
+            self.set_stroke(
+                width=stroke_width,
+                background=stroke_background,
+            )
         else:
             self.set_stroke(
                 color=stroke_color,
                 width=stroke_width,
                 opacity=stroke_opacity,
                 recurse=recurse,
+                background=stroke_background,
             )
 
         if gloss is not None:
@@ -184,6 +188,7 @@ class VMobject(Mobject):
             "fill_rgba": self.data['fill_rgba'],
             "stroke_rgba": self.data['stroke_rgba'],
             "stroke_width": self.data['stroke_width'],
+            "stroke_background": self.draw_stroke_behind_fill,
             "gloss": self.get_gloss(),
             "shadow": self.get_shadow(),
         }
@@ -424,7 +429,10 @@ class VMobject(Mobject):
 
     def set_points_smoothly(self, points, true_smooth=False):
         self.set_points_as_corners(points)
-        self.make_smooth()
+        if true_smooth:
+            self.make_smooth()
+        else:
+            self.make_approximately_smooth()
         return self
 
     def change_anchor_mode(self, mode):
