@@ -2,26 +2,28 @@ import inspect
 import random
 import platform
 import itertools as it
-import logging
 from functools import wraps
 
+import streamlit as st
 from tqdm import tqdm as ProgressDisplay
 import numpy as np
 import time
 
-from manimlib.animation.animation import prepare_animation
-from manimlib.animation.transform import MoveToTarget
-from manimlib.mobject.mobject import Point
-from manimlib.camera.camera import Camera
-from manimlib.constants import DEFAULT_WAIT_TIME
-from manimlib.mobject.mobject import Mobject
-from manimlib.scene.scene_file_writer import SceneFileWriter
-from manimlib.utils.config_ops import digest_config
-from manimlib.utils.family_ops import extract_mobject_family_members
-from manimlib.utils.family_ops import restructure_list_to_exclude_certain_family_members
-from manimlib.event_handler.event_type import EventType
-from manimlib.event_handler import EVENT_DISPATCHER
-
+from ..animation.animation import prepare_animation
+from ..animation.transform import MoveToTarget
+from ..camera.camera import Camera
+from ..constants import DEFAULT_WAIT_TIME
+from ..mobject.mobject import Mobject
+from ..mobject.mobject import Point
+from ..scene.scene_file_writer import SceneFileWriter
+from ..utils.config_ops import digest_config
+from ..utils.family_ops import extract_mobject_family_members
+from ..utils.family_ops import restructure_list_to_exclude_certain_family_members
+from ..event_handler.event_type import EventType
+from ..event_handler import EVENT_DISPATCHER
+from ..logger import log
+from ..window import Window
+from pyglet.window import key
 
 class Scene(object):
     CONFIG = {
@@ -42,15 +44,16 @@ class Scene(object):
     def __init__(self, **kwargs):
         digest_config(self, kwargs)
         if self.preview:
-            from manimlib.window import Window
+            from ..window import Window
             self.window = Window(scene=self, **self.window_config)
             self.camera_config["ctx"] = self.window.ctx
+            self.camera_config["frame_rate"] = 30  # Where's that 30 from?
         else:
             self.window = None
 
         self.camera: Camera = self.camera_class(**self.camera_config)
         self.file_writer = SceneFileWriter(self, **self.file_writer_config)
-        self.mobjects = []
+        self.mobjects = [self.camera.frame]
         self.num_plays = 0
         self.time = 0
         self.skip_time = 0
@@ -60,6 +63,8 @@ class Scene(object):
         # Items associated with interaction
         self.mouse_point = Point()
         self.mouse_drag_point = Point()
+        self.use_streamlit = False
+        self.streamlit_bar_init = False
 
         # Much nicer to work with deterministic scenes
         if self.random_seed is not None:
@@ -101,10 +106,12 @@ class Scene(object):
         # If there is a window, enter a loop
         # which updates the frame while under
         # the hood calling the pyglet event loop
+        log.info("Tips: You are now in the interactive mode. Now you can use the keyboard"
+            " and the mouse to interact with the scene. Just press `q` if you want to quit.")
         self.quit_interaction = False
         self.lock_static_mobject_data()
         while not (self.window.is_closing or self.quit_interaction):
-            self.update_frame()
+            self.update_frame(1 / self.camera.frame_rate)
         if self.window.is_closing:
             self.window.destroy()
         if self.quit_interaction:
@@ -119,16 +126,21 @@ class Scene(object):
         self.linger_after_completion = False
         self.update_frame()
 
+        # Save scene state at the point of embedding
+        self.save_state()
+
         from IPython.terminal.embed import InteractiveShellEmbed
         shell = InteractiveShellEmbed()
         # Have the frame update after each command
         shell.events.register('post_run_cell', lambda *a, **kw: self.update_frame())
         # Use the locals of the caller as the local namespace
-        # once embeded, and add a few custom shortcuts
+        # once embedded, and add a few custom shortcuts
         local_ns = inspect.currentframe().f_back.f_locals
         local_ns["touch"] = self.interact
         for term in ("play", "wait", "add", "remove", "clear", "save_state", "restore"):
             local_ns[term] = getattr(self, term)
+        log.info("Tips: Now the embed iPython terminal is open. But you can't interact with"
+            " the window directly. To do so, you need to type `touch()` or `self.interact()`")
         shell(local_ns=local_ns, stack_depth=2)
         # End scene when exiting an embed.
         raise EndSceneEarlyException()
@@ -169,6 +181,7 @@ class Scene(object):
     # Related to updating
     def update_mobjects(self, dt):
         for mobject in self.mobjects:
+            mobject: Mobject
             mobject.update(dt)
 
     def should_update_mobjects(self):
@@ -197,6 +210,7 @@ class Scene(object):
                 for family in families
             ])
             return num_families == 1
+
         return list(filter(is_top_level, mobjects))
 
     def get_mobject_family_members(self):
@@ -248,6 +262,18 @@ class Scene(object):
     def get_mobject_copies(self):
         return [m.copy() for m in self.mobjects]
 
+    def point_to_mobject(self, point, search_set=None, buff=0):
+        """
+        E.g. if clicking on the scene, this returns the top layer mobject
+        under a given point
+        """
+        if search_set is None:
+            search_set = self.mobjects
+        for mobject in reversed(search_set):
+            if mobject.is_point_touching(point, buff=buff):
+                return mobject
+        return None
+
     # Related to skipping
     def update_skipping_status(self):
         if self.start_at_animation_number is not None:
@@ -278,15 +304,18 @@ class Scene(object):
         return time_progression
 
     def get_run_time(self, animations):
-        return np.max([animation.run_time for animation in animations])
+        runtime_list = [animation.run_time for animation in animations]
+        self.st_runtime_list = runtime_list
+        return np.max(runtime_list)
 
     def get_animation_time_progression(self, animations):
         run_time = self.get_run_time(animations)
         time_progression = self.get_time_progression(run_time)
-        time_progression.set_description("".join([
+        description = "".join([
             f"Animation {self.num_plays}: {animations[0]}",
             ", etc." if len(animations) > 1 else "",
-        ]))
+        ])
+        time_progression.set_description(description)
         return time_progression
 
     def get_wait_time_progression(self, duration, stop_condition):
@@ -394,6 +423,7 @@ class Scene(object):
                 self.file_writer.end_animation()
 
             self.num_plays += 1
+
         return wrapper
 
     def lock_static_mobject_data(self, *animations):
@@ -420,11 +450,25 @@ class Scene(object):
             if animation.mobject not in self.mobjects:
                 self.add(animation.mobject)
 
+    def _check_streamlit_bar(self):
+        if self.use_streamlit:
+            if self.streamlit_bar_init is False:
+                self.placeholder = st.empty()
+                self.streamlit_bar_init = True
+            self.st_progress = self.placeholder.progress(0)
+
+    def _show_streamlit_bar(self, t, max_run_time):
+        if self.use_streamlit:
+            self.st_progress.progress(t / max_run_time)
+
     def progress_through_animations(self, animations):
         last_t = 0
+        self._check_streamlit_bar()
+        max_run_time = self.get_run_time(animations)
         for t in self.get_animation_time_progression(animations):
             dt = t - last_t
             last_t = t
+            self._show_streamlit_bar(t, max_run_time)
             for animation in animations:
                 animation.update_mobjects(dt)
                 alpha = t / animation.run_time
@@ -444,10 +488,7 @@ class Scene(object):
     @handle_play_like_call
     def play(self, *args, **kwargs):
         if len(args) == 0:
-            logging.log(
-                logging.WARNING,
-                "Called Scene.play with no animations"
-            )
+            log.warning("Called Scene.play with no animations")
             return
         animations = self.anims_from_play_args(*args, **kwargs)
         self.lock_static_mobject_data(*animations)
@@ -463,9 +504,13 @@ class Scene(object):
             self.lock_static_mobject_data()
             time_progression = self.get_wait_time_progression(duration, stop_condition)
             last_t = 0
+            if self.use_streamlit:
+                self.st_progress2 = st.progress(0)
             for t in time_progression:
                 dt = t - last_t
                 last_t = t
+                if self.use_streamlit:
+                    self.st_progress2.progress(t / duration)
                 self.update_frame(dt)
                 self.emit_frame()
                 if stop_condition is not None and stop_condition():
@@ -570,8 +615,8 @@ class Scene(object):
 
         frame = self.camera.frame
         if self.window.is_key_pressed(ord("z")):
-            factor = 1 + np.arctan(15 * offset[1])
-            frame.scale(factor, about_point=point)
+            factor = 1 + np.arctan(10 * offset[1])
+            frame.scale(1 / factor, about_point=point)
         else:
             transform = frame.get_inverse_camera_rotation_matrix()
             shift = np.dot(np.transpose(transform), offset)
@@ -583,13 +628,14 @@ class Scene(object):
         if propagate_event is not None and propagate_event is False:
             return
 
+    def _clip_control(self, symbol):
+        pass
+
     def on_key_press(self, symbol, modifiers):
         try:
-            print(symbol)
             char = chr(symbol)
         except OverflowError:
-            print(symbol)
-            print(" Warning: The value of the pressed key is too large.")
+            log.warning("The value of the pressed key is too large.")
             return
 
         event_data = {"symbol": symbol, "modifiers": modifiers}
@@ -597,15 +643,31 @@ class Scene(object):
         if propagate_event is not None and propagate_event is False:
             return
 
-        if char == "r":
+        if symbol == key.R:
             self.camera.frame.to_default_state()
-        elif char == "q":
+
+        elif symbol in (key.Q, key.TAB):  # key.APOSTROPHE,
             self.quit_interaction = True
-        elif char == " ":
-            duration = 0.1
+
+        elif symbol in (key.SPACE, key.RCTRL, key.LCTRL):
             self.pause = self.pause ^ 1
+            time_pause_start = time.time()
+            if self.pause:
+                print("\nPausing animation...")
+            if not self.pause:
+                print("\n Continue")
             while self.pause == 1:
-                self.update_frame(duration)
+                self.window: Window
+                self.window.clear()
+                if symbol == key.SPACE:
+                    self.camera.clear()
+                self.camera.capture(*self.mobjects)
+                self.window.swap_buffers()
+            time_delta = time.time() - time_pause_start
+            self.real_animation_start_time += time_delta
+
+        else:
+            self._clip_control(symbol)
 
     def on_resize(self, width: int, height: int):
         self.camera.reset_pixel_shape(width, height)
