@@ -1,10 +1,11 @@
-from manimlib.scene.scene import Scene
+from ..scene.scene import Scene
 
 
 class ThreeDScene(Scene):
     CONFIG = {
         "camera_config": {
             "samples": 4,
+            "anti_alias_width": 0,
         }
     }
 
