@@ -1,34 +1,34 @@
 import numpy as np
 
-from manimlib.animation.animation import Animation
-from manimlib.animation.creation import ShowCreation
-from manimlib.animation.creation import Write
-from manimlib.animation.fading import FadeOut
-from manimlib.animation.growing import GrowArrow
-from manimlib.animation.transform import ApplyFunction
-from manimlib.animation.transform import ApplyPointwiseFunction
-from manimlib.animation.transform import Transform
-from manimlib.constants import *
-from manimlib.mobject.coordinate_systems import Axes
-from manimlib.mobject.coordinate_systems import NumberPlane
-from manimlib.mobject.geometry import Arrow
-from manimlib.mobject.geometry import Dot
-from manimlib.mobject.geometry import Line
-from manimlib.mobject.geometry import Rectangle
-from manimlib.mobject.geometry import Vector
-from manimlib.mobject.matrix import Matrix
-from manimlib.mobject.matrix import VECTOR_LABEL_SCALE_FACTOR
-from manimlib.mobject.matrix import vector_coordinate_label
-from manimlib.mobject.mobject import Mobject
-from manimlib.mobject.svg.tex_mobject import Tex
-from manimlib.mobject.svg.tex_mobject import TexText
-from manimlib.mobject.types.vectorized_mobject import VGroup
-from manimlib.mobject.types.vectorized_mobject import VMobject
-from manimlib.scene.scene import Scene
-from manimlib.utils.rate_functions import rush_from
-from manimlib.utils.rate_functions import rush_into
-from manimlib.utils.space_ops import angle_of_vector
-from manimlib.utils.space_ops import get_norm
+from ..animation.animation import Animation
+from ..animation.creation import ShowCreation
+from ..animation.creation import Write
+from ..animation.fading import FadeOut
+from ..animation.growing import GrowArrow
+from ..animation.transform import ApplyFunction
+from ..animation.transform import ApplyPointwiseFunction
+from ..animation.transform import Transform
+from ..constants import *
+from ..mobject.coordinate_systems import Axes
+from ..mobject.coordinate_systems import NumberPlane
+from ..mobject.geometry import Arrow
+from ..mobject.geometry import Dot
+from ..mobject.geometry import Line
+from ..mobject.geometry import Rectangle
+from ..mobject.geometry import Vector
+from ..mobject.matrix import Matrix
+from ..mobject.matrix import VECTOR_LABEL_SCALE_FACTOR
+from ..mobject.matrix import vector_coordinate_label
+from ..mobject.mobject import Mobject
+from ..mobject.svg.tex_mobject import Tex
+from ..mobject.svg.tex_mobject import TexText
+from ..mobject.types.vectorized_mobject import VGroup
+from ..mobject.types.vectorized_mobject import VMobject
+from ..scene.scene import Scene
+from ..utils.rate_functions import rush_from
+from ..utils.rate_functions import rush_into
+from ..utils.space_ops import angle_of_vector
+from ..utils.space_ops import get_norm
 
 X_COLOR = GREEN_C
 Y_COLOR = RED_C
